@@ -579,6 +579,9 @@ is_fuse_file_parent (gpointer key,
   FuseFile *fuse_file = value;
   const char *parent_path = user_data;
 
+  if (!fuse_file->is_directory)
+    return FALSE;
+
   if (strcmp (parent_path, fuse_file->filename_with_root) == 0)
     return TRUE;
 
@@ -635,8 +638,7 @@ set_selection_for_clip_data_entry (GrdRdpFuseClipboard *rdp_fuse_clipboard,
           g_warning ("[RDP.CLIPRDR] Failed to convert filename. Aborting "
                      "SelectionTransfer");
           clear_entry_selection (rdp_fuse_clipboard, entry);
-
-          g_free (fuse_file);
+          fuse_file_free (fuse_file);
 
           return FALSE;
         }
@@ -653,6 +655,16 @@ set_selection_for_clip_data_entry (GrdRdpFuseClipboard *rdp_fuse_clipboard,
 
       parent = get_parent_directory (rdp_fuse_clipboard,
                                      fuse_file->filename_with_root);
+      if (!parent)
+        {
+          g_warning ("[RDP.CLIPRDR] Failed to find parent directory. Aborting "
+                     "SelectionTransfer");
+          clear_entry_selection (rdp_fuse_clipboard, entry);
+          fuse_file_free (fuse_file);
+
+          return FALSE;
+        }
+
       parent->children = g_list_append (parent->children, fuse_file);
       fuse_file->parent = parent;
 
