@@ -28,7 +28,7 @@
 #include <winpr/ssl.h>
 
 #include "grd-context.h"
-#include "grd-rdp-nvenc.h"
+#include "grd-hwaccel-nvidia.h"
 #include "grd-session-rdp.h"
 
 enum
@@ -45,15 +45,13 @@ struct _GrdRdpServer
   GList *sessions;
 
   GList *stopped_sessions;
-  guint idle_task;
+  guint cleanup_sessions_idle_id;
 
   GrdContext *context;
-#ifdef HAVE_NVENC
-  GrdRdpNvenc *rdp_nvenc;
-#endif /* HAVE_NVENC */
+  GrdHwAccelNvidia *hwaccel_nvidia;
 };
 
-G_DEFINE_TYPE (GrdRdpServer, grd_rdp_server, G_TYPE_SOCKET_SERVICE);
+G_DEFINE_TYPE (GrdRdpServer, grd_rdp_server, G_TYPE_SOCKET_SERVICE)
 
 GrdContext *
 grd_rdp_server_get_context (GrdRdpServer *rdp_server)
@@ -65,11 +63,24 @@ GrdRdpServer *
 grd_rdp_server_new (GrdContext *context)
 {
   GrdRdpServer *rdp_server;
+  GrdEglThread *egl_thread;
 
   rdp_server = g_object_new (GRD_TYPE_RDP_SERVER,
                              "context", context,
                              NULL);
 
+  egl_thread = grd_context_get_egl_thread (rdp_server->context);
+  if (egl_thread &&
+      (rdp_server->hwaccel_nvidia = grd_hwaccel_nvidia_new (egl_thread)))
+    {
+      g_message ("[RDP] Initialization of CUDA was successful");
+    }
+  else
+    {
+      g_debug ("[RDP] Initialization of CUDA failed. "
+               "No hardware acceleration available");
+    }
+
   return rdp_server;
 }
 
@@ -84,7 +95,7 @@ static gboolean
 cleanup_stopped_sessions_idle (GrdRdpServer *rdp_server)
 {
   grd_rdp_server_cleanup_stopped_sessions (rdp_server);
-  rdp_server->idle_task = 0;
+  rdp_server->cleanup_sessions_idle_id = 0;
 
   return G_SOURCE_REMOVE;
 }
@@ -98,9 +109,9 @@ on_session_stopped (GrdSession   *session,
   rdp_server->stopped_sessions = g_list_append (rdp_server->stopped_sessions,
                                                 session);
   rdp_server->sessions = g_list_remove (rdp_server->sessions, session);
-  if (!rdp_server->idle_task)
+  if (!rdp_server->cleanup_sessions_idle_id)
     {
-      rdp_server->idle_task =
+      rdp_server->cleanup_sessions_idle_id =
         g_idle_add ((GSourceFunc) cleanup_stopped_sessions_idle,
                     rdp_server);
     }
@@ -116,14 +127,10 @@ on_incoming (GSocketService    *service,
   g_debug ("New incoming RDP connection");
 
   if (!(session_rdp = grd_session_rdp_new (rdp_server, connection,
-#ifdef HAVE_NVENC
-                                           rdp_server->rdp_nvenc,
-#endif /* HAVE_NVENC */
-                                           0)))
+                                           rdp_server->hwaccel_nvidia)))
     return TRUE;
 
   rdp_server->sessions = g_list_append (rdp_server->sessions, session_rdp);
-  grd_context_add_session (rdp_server->context, GRD_SESSION (session_rdp));
 
   g_signal_connect (session_rdp, "stopped",
                     G_CALLBACK (on_session_stopped),
@@ -149,11 +156,23 @@ grd_rdp_server_start (GrdRdpServer  *rdp_server,
   return TRUE;
 }
 
-static void
-stop_and_unref_session (GrdSession *session)
+void
+grd_rdp_server_stop (GrdRdpServer *rdp_server)
 {
-  grd_session_stop (session);
-  g_object_unref (session);
+  g_socket_service_stop (G_SOCKET_SERVICE (rdp_server));
+  g_socket_listener_close (G_SOCKET_LISTENER (rdp_server));
+
+  while (rdp_server->sessions)
+    {
+      GrdSession *session = rdp_server->sessions->data;
+
+      grd_session_stop (session);
+    }
+
+  g_clear_handle_id (&rdp_server->cleanup_sessions_idle_id, g_source_remove);
+  grd_rdp_server_cleanup_stopped_sessions (rdp_server);
+
+  g_clear_object (&rdp_server->hwaccel_nvidia);
 }
 
 static void
@@ -198,26 +217,11 @@ grd_rdp_server_dispose (GObject *object)
 {
   GrdRdpServer *rdp_server = GRD_RDP_SERVER (object);
 
-#ifdef HAVE_NVENC
-  g_clear_object (&rdp_server->rdp_nvenc);
-#endif /* HAVE_NVENC */
+  g_assert (!rdp_server->sessions);
+  g_assert (!rdp_server->cleanup_sessions_idle_id);
+  g_assert (!rdp_server->stopped_sessions);
 
-  if (rdp_server->idle_task)
-    {
-      g_source_remove (rdp_server->idle_task);
-      rdp_server->idle_task = 0;
-    }
-
-  if (rdp_server->stopped_sessions)
-    {
-      grd_rdp_server_cleanup_stopped_sessions (rdp_server);
-    }
-  if (rdp_server->sessions)
-    {
-      g_list_free_full (rdp_server->sessions,
-                        (GDestroyNotify) stop_and_unref_session);
-      rdp_server->sessions = NULL;
-    }
+  g_assert (!rdp_server->hwaccel_nvidia);
 
   G_OBJECT_CLASS (grd_rdp_server_parent_class)->dispose (object);
 }
@@ -238,22 +242,6 @@ grd_rdp_server_init (GrdRdpServer *rdp_server)
    * Run the primitives benchmark here to save time, when initializing a session
    */
   primitives_get ();
-
-#ifdef HAVE_NVENC
-  rdp_server->rdp_nvenc = grd_rdp_nvenc_new ();
-  if (rdp_server->rdp_nvenc)
-    {
-      g_debug ("[RDP] Initialization of NVENC was successful");
-    }
-  else
-    {
-      g_message ("[RDP] Initialization of NVENC failed. "
-                 "No hardware acceleration available");
-    }
-#else
-  g_message ("[RDP] RDP backend is built WITHOUT support for NVENC and CUDA. "
-             "No hardware acceleration available");
-#endif /* HAVE_NVENC */
 }
 
 static void
