@@ -1659,8 +1659,9 @@ rdp_peer_capabilities (freerdp_peer *peer)
       session_rdp->monitor_config->monitor_count = 1;
     }
 
-  if (rdp_settings->SupportGraphicsPipeline || rdp_settings->RemoteFxCodec ||
-      rdp_settings->NSCodec)
+  if ((rdp_settings->SupportGraphicsPipeline || rdp_settings->RemoteFxCodec ||
+       rdp_settings->NSCodec) &&
+      rdp_settings->ColorDepth != 32)
     {
       g_debug ("[RDP] Fixing invalid colour depth set by client");
       rdp_settings->ColorDepth = 32;
@@ -1825,7 +1826,8 @@ rdp_peer_context_free (freerdp_peer   *peer,
   if (!rdp_peer_context)
     return;
 
-  g_clear_pointer (&rdp_peer_context->vcm, WTSCloseServer);
+  if (rdp_peer_context->vcm != INVALID_HANDLE_VALUE)
+    g_clear_pointer (&rdp_peer_context->vcm, WTSCloseServer);
 
   if (rdp_peer_context->encode_stream)
     {
