@@ -518,7 +518,11 @@ check_rfb_password (rfbClientPtr  rfb_client,
   password = grd_settings_get_vnc_password (settings, &error);
   if (!password)
     {
-      g_warning ("Couldn't retrieve VNC password: %s", error->message);
+      if (error)
+        g_warning ("[VNC] Couldn't retrieve VNC password: %s", error->message);
+      else
+        g_message ("[VNC] Password is not set, denying client");
+
       return FALSE;
     }
 
@@ -654,7 +658,11 @@ grd_session_vnc_attach_source (GrdSessionVnc *session_vnc)
 static void
 grd_session_vnc_detach_source (GrdSessionVnc *session_vnc)
 {
-  g_clear_pointer (&session_vnc->source, g_source_destroy);
+  if (!session_vnc->source)
+    return;
+
+  g_source_destroy (session_vnc->source);
+  g_clear_pointer (&session_vnc->source, g_source_unref);
 }
 
 GrdSessionVnc *
