@@ -135,13 +135,16 @@ grd_settings_get_rdp_username (GrdSettings  *settings,
                                                     NULL, error,
                                                     NULL);
   if (!credentials_string)
+    return NULL;
+
+  credentials = g_variant_parse (NULL, credentials_string, NULL, NULL, NULL);
+  if (!credentials)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                   "Credentials not set");
+                   "Unable to parse credentials");
       return NULL;
     }
 
-  credentials = g_variant_parse (NULL, credentials_string, NULL, NULL, NULL);
   g_variant_lookup (credentials, "username", "s", &username);
   if (!username)
     {
@@ -170,13 +173,16 @@ grd_settings_get_rdp_password (GrdSettings  *settings,
                                                     NULL, error,
                                                     NULL);
   if (!credentials_string)
+    return NULL;
+
+  credentials = g_variant_parse (NULL, credentials_string, NULL, NULL, NULL);
+  if (!credentials)
     {
       g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                   "Credentials not set");
+                   "Unable to parse credentials");
       return NULL;
     }
 
-  credentials = g_variant_parse (NULL, credentials_string, NULL, NULL, NULL);
   g_variant_lookup (credentials, "password", "s", &password);
   if (!password)
     {
@@ -203,11 +209,7 @@ grd_settings_get_vnc_password (GrdSettings  *settings,
                                           NULL, error,
                                           NULL);
   if (!password)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                   "Password not set");
-      return NULL;
-    }
+    return NULL;
 
   return password;
 }
